//This file is part of Furius Iso Mount. Copyright 2008 Dean Harris (marcus_furius@hotmail.com)
//
//    Furius Iso Mount is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    Furius Iso Mount is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with Furius Iso Mount.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.IO;
using FuriusIsoMount;
using System.Diagnostics;
using Gtk;
using System.Threading;

namespace FuriusIsoMount
{
	
	/// <summary>
	/// Mounts an Image
	/// </summary>
	public static class ImageAction
	{
				
		/// <summary>
	    /// Mounts an Image
	    /// </summary>
		/// <param name="imageFile">The filename.</param>
        /// <param name="treeStore">The TreeStore used to store the current mounts</param>
		public static void Mount(string imageFile, TreeStore treeStore, ComboBoxEntry comboBoxEntry, string[] history, bool isFuse)
		{
			FileInfo file = new FileInfo(imageFile);
			//Replace any characters that could screw us up.
			string mountLocation = Globals.HomeDirectory + file.Name.Replace(' ', '_');
			mountLocation = mountLocation.Replace('.', '_');
			try
			{
				// If an image is already mounted this will throw an IOException
				if (Directory.Exists(mountLocation))
				{
					Directory.Delete(mountLocation);
				}
				Directory.CreateDirectory(mountLocation);
				
				if (isFuse)
				{				
					// Mount the image, writing the executed command to the log
					Log.Write(Globals.MountLog, "fuseiso '" + imageFile + "' '"+ mountLocation +"'");
					ExternalProcess("fuseiso", "'" + imageFile + "' '"+ mountLocation +"'", false);
					//Write an entry with instructions on how to remove manually if needed
					Log.Write(Globals.MountLog, 
				          string.Format(LocalizableResources.LogEntryMountSuccess, imageFile, mountLocation));
					
				}
				else
				{
					Log.Write(Globals.MountLog, "gksu -u root \"mount -t iso9660 -o loop '" + imageFile + "' '"+ mountLocation +"'\"");
					//Use Loopback
					ExternalProcess("gksu", "-u root \"mount -t iso9660 -o loop '" + imageFile + "' '"+ mountLocation +"'\"", false);
					Log.Write(Globals.MountLog, 
				          string.Format(LocalizableResources.LogEntryMountSuccessLoop, imageFile, mountLocation));
				}
											
				
				
				// Add mount point and image to treeStore
				treeStore.AppendValues (mountLocation, imageFile, isFuse.ToString());
				
				// Add the image to the history list
				AddItemToHistory(imageFile, comboBoxEntry, history);

			}
			catch (UnauthorizedAccessException exception)
			{
				Log.Write(Globals.MountLog, exception.Message);
				throw;
			}
			catch (IOException exception)
			{
				Log.Write(Globals.MountLog, exception.Message);
				throw;
			}
		}

		/// <summary>
		/// Add the image to the history list
		/// </summary>
		/// <param name="imageFile">A <see cref="System.String"/></param>
		/// <param name="comboBoxEntry">A <see cref="ComboBoxEntry"/></param>
		/// <param name="history">A <see cref="StringCollection"/></param>
		private static void AddItemToHistory(string imageFile, ComboBoxEntry comboBoxEntry, string[] history)
		{
			// Clear the comboBox			
			while(comboBoxEntry.Model.IterNChildren() != 0)
			{
				comboBoxEntry.RemoveText(0);
			}
						
			// If the image already exists in the list, remove it. We'll be adding it
			// to the top of the list.
			for (int i = 0; i < history.Length; i ++)
			{
				if (history[i] == imageFile)
				{
					Globals.RemoveFromArray(history, i);
				}
			}
			
			// Add the image to the top
			Globals.InsertIntoArray(history, imageFile, 0);
								
			// Finally rebuild the menu items and create our history
			using (StreamWriter streamWriter = new StreamWriter(Globals.HistoryList, false))
			{
				foreach(string historyItem in history)
				{
					if (historyItem != null)
					{
						comboBoxEntry.AppendText(historyItem);
						try
						{
							streamWriter.WriteLine(historyItem);
						}
						catch (IOException exception)
						{
							Log.Write(Globals.MountLog, exception.Message);
							throw;
						}
					}
				}
			}
			
		}
		
		/// <summary>
	    /// Unmounts an Image
	    /// </summary>
		/// <param name="mountPoint">The mount point.</param>
        /// <param name="treeStore">The TreeStore used to store the current mounts</param>
		/// <param name="iter">The TreeIter to remove from the TreeStore</param>
		public static void Unmount(string mountPoint, TreeStore treeStore, TreeIter iter, bool isShutDown, bool isFuse)
		{
			try
			{
				// If the directory doesn't exist, it may have been
				// unmounted externally so just remove the TreeIter
				// from the TreeStore
				if (Directory.Exists(mountPoint))
				{
					// Start as a named process so we can ensure it
					// has completed before trying to remove the directory
					
					if (isFuse)
					{
						ExternalProcess("fusermount", "-u '"+ mountPoint +"'", true);
					}
					else
					{
						ExternalProcess("gksu", "-u root \"umount '"+ mountPoint +"'\"", true);
					}
					
					Thread.Sleep(500); // Patience my pretty!
					// When unmounting has completed delete the directory
					Directory.Delete(mountPoint);
					Log.Write(Globals.MountLog, 
				          string.Format(LocalizableResources.LogEntryUnmountSuccess, mountPoint));
				}
				if (!isShutDown)
				{
					// Remove mount point info from the TreeStore
					treeStore.Remove(ref iter);
				}
			}
			catch (UnauthorizedAccessException exception)
			{
				Log.Write(Globals.MountLog, exception.Message);
				throw;
			}
			catch (IOException exception)
			{
				Log.Write(Globals.MountLog, exception.Message);
				throw;
			}
		}
		
		/// <summary>
	    /// Opens Nautilus at the selected mount point
	    /// </summary>
		/// <param name="mountPoint">The mount point.</param>
		public static void Browse(string mountPoint)
		{
			 Process.Start("nautilus --browser '"+ mountPoint +"'");
			 Log.Write(Globals.MountLog, 
				          string.Format("{0}: nautilus --browser '{1}'", 
			                        LocalizableResources.LogEntryCommandAttempted, mountPoint));
		}
		
		/// <summary>
	    /// Starts the Nautilus CD burning wizard
	    /// </summary>
		/// <param name="imageFile">The Image file to burn</param>
		public static void BurnNautilus(string imageFile)
		{
			Process.Start("nautilus-cd-burner --source-iso='"+ imageFile +"'");
			Log.Write(Globals.MountLog, 
				          string.Format("{0}: nautilus-cd-burner --source-iso='{1}'", 
			                        LocalizableResources.LogEntryCommandAttempted, imageFile));
		}
		
		/// <summary>
	    /// Starts the Brasero CD burning wizard
	    /// </summary>
		/// <param name="imageFile">The Image file to burn</param>
		public static void BurnBrasero(string imageFile)
		{
			Process.Start("brasero --image '"+ imageFile +"'");
			Log.Write(Globals.MountLog, 
				          string.Format("{0}: brasero --image '{1}'", 
			                        LocalizableResources.LogEntryCommandAttempted, imageFile));
		}
		
		
		/// <summary>
	    /// Starts a process with the passed command and arguments, while loging the output and errors
	    /// </summary>
	    /// <remarks>
	    /// This could be used for other command, ofcourse, but the 
	    /// fuse and mount processes are the only commandline processes started.
	    /// </remarks>
		/// <param name="command">The command to execute</param>
        /// <param name="arguments">The command arguments</param>
		/// <param name="isLongProcess">A boolean indicating if we should wait for the process complete</param>
		private static void ExternalProcess(string command, string arguments, bool isLongProcess)
		{
			Process externalProcess = new Process();
			externalProcess.StartInfo.UseShellExecute = false;
		    externalProcess.StartInfo.RedirectStandardOutput = true;
		    externalProcess.StartInfo.RedirectStandardError = true;
		    externalProcess.StartInfo.CreateNoWindow = true;
			externalProcess.StartInfo.FileName = command;
			externalProcess.StartInfo.Arguments = arguments;
		    externalProcess.Start();
			
			if (isLongProcess)
			{
				while (!externalProcess.HasExited)
		        {
		            Thread.Sleep(500); // sleep  			
		        }
			}

			string output = externalProcess.StandardOutput.ReadToEnd();	
			if (!string.IsNullOrEmpty(output))
			{
				Log.Write(Globals.MountLog,
			          output);
			}
			
			string error = externalProcess.StandardError.ReadToEnd();
			if (!string.IsNullOrEmpty(error))
			{
				Log.Write(Globals.MountLog,
			          error);
				MessageBox.Show(null, error, MessageType.Warning);
			}
		}
		
	}
}
